*        File supl.f
*     Miscellaneous subroutines
*     
*|||||||     Initial state    ||||||||||||||||||||||||||||||||||||||||
*
*================ CUBIC =================================================
*                                    
*   Set molecules on a cubic lattice
*
      SUBROUTINE CUBIC(X,Y,Z,BOXL,BOYL,BOZL,NOP,ICELL)
      IMPLICIT real*8 (A-H,O-Z)
*
      DIMENSION X(*),Y(*),Z(*)
*
      FNOP     = DFLOAT(NOP) 
	if(ICELL.eq.1)then
	  FAC=2.
	else
	  FAC=1.
	end if          
	SIZE	 = (BOXL*BOYL*BOZL/FNOP/FAC)**(1.d0/3.d0)
      NRX      = BOXL/SIZE
      NRY      = BOYL/SIZE
      NRZ      = BOZL/SIZE
 10 	write(*,*)NRX,NRY,NRZ
	if(NRX*NRY*NRZ.lt.NOP)NRX=NRX+1
	if(NRX*NRY*NRZ.lt.NOP)NRY=NRY+1
	if(NRX*NRY*NRZ.lt.NOP)then
        NRZ=NRZ+1
	  go to 10
	end if
      NRCUBE   = NRX*NRY*NRZ
      NEMPTY   = NRCUBE-NOP
      L        = 0
      DO 100 I = 1,NRX
      DO 100 J = 1,NRY
      DO 100 K = 1,NRZ
      L        = L+1
      X(L)     = (DFLOAT(I)-0.5)*BOXL/NRX-0.5*BOXL
      Y(L)     = (DFLOAT(J)-0.5)*BOYL/NRY-0.5*BOYL
      Z(L)     = (DFLOAT(K)-0.5)*BOZL/NRZ-0.5*BOZL 
	if(ICELL.eq.1.and.(abs(X(L))+abs(Y(L))+abs(Z(L))).gt.0.75*BOXL)
     +   L=L-1
	if(L.ge.NOP)go to 110
  100 CONTINUE 
	if(L.lt.NOP)then
	  NRX=NRX+1
	  go to 10
	end if
  110 NSKIP    = 0
      IF(NEMPTY.NE.0) NSKIP=NOP/NEMPTY+1
      PRINT "(/1X,'*** LATTICE: ',3I4,' SITES/EDGE  -> ',I4,
     X' SITES TOTALLY ( ',I3,' VACANT  - WITH INTERVALL OF ',I3,' )'/)"
     X,NRX,NRY,NRZ,NRCUBE,NEMPTY,NSKIP
*
      DO I = 1,NOP
        call PBC(X(I),Y(I),Z(I))
	end do
*                           
	call MIXTURE(X,Y,Z,NOP)
*
      RETURN
      END
*
*=============== FCC ===================================================
*                                   
*  Set molecules on the FCC lattice
      SUBROUTINE FCC(X,Y,Z,BOXL,BOYL,BOZL,NSP,ICELL)
      IMPLICIT real*8 (A-H,O-Z)
      DIMENSION X(NSP),Y(NSP),Z(NSP)
      DIMENSION XB(4),YB(4),ZB(4) 
      DATA XB/0.5,-.5,0.5,-.5/,YB/0.5,-.5,-.5,0.5/,ZB/0.5,0.5,-.5,-.5/
C     SET UP STARTING FCC LATTICE 
      HBOXL	= 0.5*BOXL 
	VOL=BOXL*BOYL*BOZL
	if(ICELL.eq.1)then
	  FAC=2.
	else
	  FAC=1.
	end if  
	SCELL=(VOL/NSP)**0.333333333333
	NCX=BOXL/SCELL+1        
	NCY=BOYL/SCELL+1        
	NCZ=BOZL/SCELL+1 
 5    continue
	if(NCX*NCY*NCZ.lt.NSP)then
	  NCX=NCX+1
	  go to 5
	end if        
 10   DRINK	= 0.01*HBOXL/NCX
      FACT   = HBOXL/DFLOAT(NCX)
      SHIFX  = BOXL/DFLOAT(NCX)
      SHIFY  = BOYL/DFLOAT(NCY)
      SHIFZ  = BOZL/DFLOAT(NCZ)
      BASE   = HBOXL*(-1.D0+1./DFLOAT(NCX))
      IL     = 0
      DO 20 IB = 1,4
        ZS       = BASE+ZB(IB)*FACT
        DO 19 IZ = 1,NCZ
          YS       = BASE+YB(IB)*FACT
          DO 18 IY = 1,NCY
            XS       = BASE+XB(IB)*FACT
            DO 17 IX = 1,NCX
              IL       = IL+1
              X(IL)    = XS
              Y(IL)    = YS
              Z(IL)    = ZS
		if(ICELL.eq.1.and.(abs(X(IL))+abs(Y(IL))+abs(Z(IL))).gt.
     +          0.75*BOXL)IL=IL-1
              if(IL.ge.NSP)go to 24
              XS       = XS+SHIFX
   17       CONTINUE
            YS       = YS+SHIFY
   18     CONTINUE
          ZS       = ZS+SHIFZ
   19   CONTINUE
   20 CONTINUE  
	if(IL.lt.NSP)then
	  IL=0
	  NCY=NCY+1
	  go to 10
	end if
*
   24 DO I     = 1,NSP
        FACT     = (2.D0*(RANF()-0.5D0))*DRINK
        X(I)     = X(I)+FACT*X(I)
        FACT     = (2.D0*(RANF()-0.5D0))*DRINK
        Y(I)     = Y(I)+FACT*Y(I)
        FACT     = (2.D0*(RANF()-0.5D0))*DRINK
        Z(I)     = Z(I)+FACT*Z(I)
      END DO
*
      do I=1,NSP
        call PBC(X(I),Y(I),Z(I))
      end do
*
      PRINT *,'    FCC LATTICE CREATED FOR ',NSP,' POINTS'
*                           
	call MIXTURE(X,Y,Z,NSP)
*
      RETURN
      END
*
*=================== DNASOL ====================================
* 
*  Put one large molecule in a cylindrical hole along Z axis and
*  distribute other molecules around it
*
	subroutine DNASOL
	include "prcm.h"
	dimension XRS(NPART),YRS(NPART),ZRS(NPART)
	NOPSM=0
	do ITYP=1,NTYPES
	  if(IINIT(ITYP).ne.1)NOPSM=NOPSM+NSPEC(ITYP)
	end do
	VOLAV=VOL-BOZL*PI*RDNA**2
	if(TASKID.eq.MAST)
     +write(*,*)' Distribute ',NOPSM,' molecules in ',VOLAV,' A**3'
	SHAG=(VOLAV/NOPSM)**0.333333333 
	IAT=0 
	NSX=BOXL/SHAG+0.7
	NSY=BOYL/SHAG+0.7
	NSZ=BOZL/SHAG+0.5
 10	NCOUNT=0 
	do IX=1,NSX
	  XX=-HBOXL+BOXL*(dfloat(IX)-0.6)/NSX
	  do IY=1,NSY
	    YY=-HBOYL+BOYL*(dfloat(IY)-0.45)/NSY 
	    RR=sqrt(XX**2+YY**2)
	    if(RR.gt.RDNA)then
	      do IZ=1,NSZ
	        NCOUNT=NCOUNT+1
	        XRS(NCOUNT)=XX
	 	YRS(NCOUNT)=YY
	        ZRS(NCOUNT)=-HBOZL+BOZL*(dfloat(IZ)-0.45)/NSZ
	        if(NCOUNT.ge.NOPSM)go to 20
            end do
	    end if
	  end do
	end do
	if(NCOUNT.le.NOPSM)then
	  IAT=IAT+1
	  if(mod(IAT,3).eq.0)then
	    NSX=NSX+1
	    if(TASKID.eq.MAST)write(*,*)' increase NSX to ',NSX
	  else if(mod(IAT,3).eq.1)then
	    NSY=NSY+1
	    if(TASKID.eq.MAST)write(*,*)' increase NSY to ',NSY
	  else
	    NSZ=NSZ+1
	    if(TASKID.eq.MAST)write(*,*)' increase NSZ to ',NSZ
	  end if
	  go to 10
	end if
*  peremeshivanie
 20	call MIXTURE(XRS,YRS,ZRS,NOPSM)
	J=0
	JMOL=0
	do ITYP=1,NTYPES 
	  IBEG=IADDR(ITYP)+1
	  IEND=IADDR(ITYP+1)
	  if(IINIT(ITYP).eq.1)then
	    if(NSPEC(ITYP).gt.1)stop 
     +' only 1 molecule of such type is allowed'
	    JMOL=JMOL+1
	    X(JMOL)=0.d0
	    Y(JMOL)=0.d0
	    Z(JMOL)=0.d0
	    if(IPRINT.ge.8)write(*,*)
     +' put molecule ',JMOL,' at zero ref.point' 
	  else
	    do I=IBEG,IEND 
	      J=J+1
	      JMOL=JMOL+1
	      X(JMOL)=XRS(J)	
	      Y(JMOL)=YRS(J)	
	      Z(JMOL)=ZRS(J)	
	    end do
	  end if
	end do	      	          
 	if(TASKID.eq.MAST)write(*,*)JMOL,' molecules are distributed' 
	if(IPRINT.ge.8)then
	  WRITE(*,*)' COM of molecules:'
	  DO ityp=1,ntypes	
	    IBEG=IADDR(ITYP)+1
	    IEND=IADDR(ITYP+1)
          do I=IBEG,IEND
	       write(*,'(2i5,3f14.5)')ITYP,I,X(I),Y(I),Z(I)
	    end do
	  end do
	end if
	return
	end
*
*=================== SPHOLE ====================================
*
*  Put one large molecule in a spherical hole and
*  distribute other molecules around it
*
	subroutine SPHOLE
	include "prcm.h"
	dimension XRS(NPART),YRS(NPART),ZRS(NPART)
	NOPSM=0
	do ITYP=1,NTYPES
	  if(IINIT(ITYP).ne.1)NOPSM=NOPSM+NSPEC(ITYP)
	end do
	VOLAV=VOL-4.d0*PI*RDNA**3/3.d0
	if(TASKID.eq.MAST)
     +write(*,*)' Distribute ',NOPSM,' molecules in ',VOLAV,' A**3'
	SHAG=(VOLAV/NOPSM)**0.333333333 
	IAT=0 
	NSX=BOXL/SHAG+0.7
	NSY=BOYL/SHAG+0.7
	NSZ=BOZL/SHAG+0.5
 10	NCOUNT=0 
	do IX=1,NSX
	  XX=-HBOXL+BOXL*(dfloat(IX)-0.6)/NSX
	  do IY=1,NSY
	    YY=-HBOYL+BOYL*(dfloat(IY)-0.45)/NSY
	    do IZ=1,NSZ 
	      ZZ=-HBOZL+BOZL*(dfloat(IZ)-0.55)/NSZ
	      RR=sqrt(XX**2+YY**2+ZZ**2)
	      if(RR.gt.RDNA.and.(.not.LOCT.or.
     +abs(XX)+abs(YY)+abs(ZZ).lt.0.75*BOXL))then
	        NCOUNT=NCOUNT+1
	        XRS(NCOUNT)=XX
	 	YRS(NCOUNT)=YY
	        ZRS(NCOUNT)=ZZ
	        if(NCOUNT.ge.NOPSM)go to 20
            end if
	    end do
	  end do
	end do
	if(NCOUNT.le.NOPSM)then
	  IAT=IAT+1
	  if(mod(IAT,3).eq.0)then
	    NSX=NSX+1
	    if(TASKID.eq.MAST)write(*,*)' increase NSX to ',NSX
	  else if(mod(IAT,3).eq.1)then
	    NSY=NSY+1
	    if(TASKID.eq.MAST)write(*,*)' increase NSY to ',NSY
	  else
	    NSZ=NSZ+1
	    if(TASKID.eq.MAST)write(*,*)' increase NSZ to ',NSZ
	  end if
	  go to 10
	end if
*  blandning
 20	call MIXTURE(XRS,YRS,ZRS,NOPSM)
*  put fixed molecules on their places
	J=0
	JMOL=0
	do ITYP=1,NTYPES 
	  IBEG=IADDR(ITYP)+1
	  IEND=IADDR(ITYP+1)
	  if(IINIT(ITYP).eq.1)then
	    if(NSPEC(ITYP).gt.1)stop 
     +' only 1 molecule of such type is allowed'
	    JMOL=JMOL+1
	    X(JMOL)=0.d0
	    Y(JMOL)=0.d0
	    Z(JMOL)=0.d0
	    if(IPRINT.ge.8)write(*,*)
     +' put molecule ',JMOL,' at zero ref.point' 
	  else
	    do I=IBEG,IEND 
	      J=J+1
	      JMOL=JMOL+1
	      X(JMOL)=XRS(J)	
	      Y(JMOL)=YRS(J)	
	      Z(JMOL)=ZRS(J)	
	    end do
	  end if
	end do	      	          
 	if(TASKID.eq.MAST)write(*,*)JMOL,' molecules are distributed' 
	if(IPRINT.ge.8)then
	  WRITE(*,*)' COM of molecules:'
	  DO ityp=1,ntypes	
	    IBEG=IADDR(ITYP)+1
	    IEND=IADDR(ITYP+1)
          do I=IBEG,IEND
	       write(*,'(2i5,3f14.5)')ITYP,I,X(I),Y(I),Z(I)
	    end do
	  end do
	end if
	return
	end     
*
*===================== MIXTURE ===================================
* 
*  Mixture molecules to avoid artificial correlations
*
	subroutine MIXTURE(X,Y,Z,NOP)
	real*8 X(NOP),Y(NOP),Z(NOP)
*
*  making mixture
*
 	do ICT=2,7
	  if(mod(ICT,2).eq.0)then
	    J=NOP
	    do I=1,NOP,ICT
	      XX=X(I)
	      YY=Y(I)
	      ZZ=Z(I)
	      X(I)=X(J)
	      Y(I)=Y(J)
	      Z(I)=Z(J)
	      X(J)=XX
	      Y(J)=YY
	      Z(J)=ZZ
	      J=J-1
	    end do
	  else
	    J=1
	    do I=1,NOP,ICT
	      XX=X(I)
	      YY=Y(I)
	      ZZ=Z(I)
	      X(I)=X(J)
	      Y(I)=Y(J)
	      Z(I)=Z(J)
	      X(J)=XX
	      Y(J)=YY
	      Z(J)=ZZ
	      J=J+1
	    end do
	  end if
      end do
	return
	end
*
*=============== RANQ =========================================
*
      SUBROUTINE RANQ(QP)
      IMPLICIT real*8 (A-H,O-Z)
      DIMENSION QP(4)
*
      PI=3.14159D0
      A=PI*RANF()/2.D0
      B=PI*RANF()
      C=PI*RANF()
      QP(1)=DSIN(A)*DSIN(B-C)
      QP(2)=DSIN(A)*DCOS(B-C)
      QP(3)=DCOS(A)*DSIN(B+C)
      QP(4)=DCOS(A)*DCOS(B+C)
      RETURN
      END
*
*=============== ROTATE ==============================================
*
      SUBROUTINE ROTATE(Q,X,Y,Z)
      IMPLICIT real*8 (A-H,O-Z)
      DIMENSION Q(4)
      SIGN = -1.D0
      AA   = Q(1)*Q(1)
      BB   = Q(2)*Q(2)
      CC   = Q(3)*Q(3)
      DD   = Q(4)*Q(4)
      AB   = Q(1)*Q(2)
      AC   = Q(1)*Q(3)
      AD   = Q(1)*Q(4)*SIGN
      BC   = Q(2)*Q(3)
      BD   = Q(2)*Q(4)*SIGN
      CD   = Q(3)*Q(4)*SIGN
      RX   = X*(-AA+BB-CC+DD)+Y*(+CD-AB)*2.D0 +Z*(+BC+AD)*2.D0
      RY   = X*(-AB-CD)*2.D0 +Y*(+AA-BB-CC+DD)+Z*(+BD-AC)*2.D0
      RZ   = X*(+BC-AD)*2.D0 +Y*(-AC-BD)*2.D0 +Z*(-AA-BB+CC+DD)
      X    = RX
      Y    = RY
      Z    = RZ
      RETURN
      END
*
*=============== COMVEL ========================================
*
      SUBROUTINE COMVEL(VX,VY,VZ,TEMP,NTOT)
      IMPLICIT real*8 (A-H,O-Z)
      DIMENSION VX(*),VY(*),VZ(*)
*
      RTEMP   = DSQRT(TEMP)            
      DO N    = 1,NTOT
      VX(N)   = RTEMP*GAUSS(DUMMY)
      VY(N)   = RTEMP*GAUSS(DUMMY)
      VZ(N)   = RTEMP*GAUSS(DUMMY)
      END DO! OF N
*
      SUMX    = 0.D0
      SUMY    = 0.D0
      SUMZ    = 0.D0
      DO N    = 1,NTOT
      SUMX    = SUMX+VX(N)
      SUMY    = SUMY+VY(N)
      SUMZ    = SUMZ+VZ(N)
      END DO! OF N
      SUMX    = SUMX/DFLOAT(NTOT)
      SUMY    = SUMY/DFLOAT(NTOT)
      SUMZ    = SUMZ/DFLOAT(NTOT)
*
      DO N    = 1,NTOT
      VX(N)   = VX(N)-SUMX
      VY(N)   = VY(N)-SUMY
      VZ(N)   = VZ(N)-SUMZ
      END DO! OF N
*
      RETURN
      END
*
*============ GAUSS ===============================================
*
      real*8 FUNCTION GAUSS(DUMMY)
      IMPLICIT real*8 (A-H,O-Z)
      PARAMETER (A1=3.949846138,A3=0.252408784)
      PARAMETER (A5=0.076542912,A7=0.008355968)
      PARAMETER (A9=0.029899776)
      SUM   = 0.0
      DO I  = 1,12
      SUM   = SUM + RANF()
      END DO                   
      R     = (SUM-6.0)/4.0
      R2    = R*R
      GAUSS = ((((A9*R2+A7)*R2+A5)*R2+A3)*R2+A1)
      RETURN
      END
*
*============================== RANF =========================
*
      real*8 FUNCTION RANF()
      IMPLICIT real*8 (A-H,O-Z)
      INTEGER L,C,M,SEED
      PARAMETER (L=1029,C=221591,M=1048576)
      SAVE SEED
      DATA SEED /0/
      SEED    = MOD(SEED*L+C,M)
      RANF    = DFLOAT(SEED)/M
      RETURN	
      END                                            
*  
*||||||||||||||||<<   Scaling   >>||||||||||||||||||||||||||||
*
*
*=============== SCALEV ==========================================
*
      SUBROUTINE SCALEV(VX,VY,VZ,FTMP,N)
      IMPLICIT real*8 (A-H,O-Z)
*
      DIMENSION VX(*),VY(*),VZ(*)
      DO I  = 1,N
      VX(I) = VX(I)*FTMP
      VY(I) = VY(I)*FTMP
      VZ(I) = VZ(I)*FTMP
      END DO
      END
*
*=============== SCLFRC ===================================
*
      SUBROUTINE SCLFRC(DONE,NNN)
      include "prcm.h"
*
      LOGICAL DONE
*
      IF(.NOT.LSCFT) RETURN
*
      GO TO (1000,2000,3000),NNN
*
      STOP '!!! WRONG OPTION IN SCLFRC !!!'
*
 1000 CONTINUE
      DONE      = .FALSE.
      DO I      = 1,NSTOT
      FSTERM    = HX(I)**2+HY(I)**2+HZ(I)**2
      FSTERM    = DSQRT(FSTERM)*FTSCL
      IF(FSTERM.GT.1.D0) THEN
      HX(I)     = HX(I)/FSTERM
      HY(I)     = HY(I)/FSTERM
      HZ(I)     = HZ(I)/FSTERM
      DONE      = .TRUE.
      END IF
      END DO! OF I
      RETURN
*
 2000 CONTINUE
      DONE      = .FALSE.
      DO I      = 1,NSTOT
      FSTERM    = GX(I)**2+GY(I)**2+GZ(I)**2
      FSTERM    = DSQRT(FSTERM)*FTSCL
      IF(FSTERM.GT.1.D0) THEN
      GX(I)     = GX(I)/FSTERM
      GY(I)     = GY(I)/FSTERM
      GZ(I)     = GZ(I)/FSTERM
      DONE      = .TRUE.
      END IF
      END DO! OF I
      RETURN
*
 3000 CONTINUE
      DONE      = .FALSE.
      DO I      = 1,NSTOT
      FSTERM    = FX(I)**2+FY(I)**2+FZ(I)**2
      FSTERM    = DSQRT(FSTERM)*FTSCL
      IF(FSTERM.GT.1.D0) THEN
      FX(I)     = FX(I)/FSTERM
      FY(I)     = FY(I)/FSTERM
      FZ(I)     = FZ(I)/FSTERM
      DONE      = .TRUE.
      END IF
      END DO! OF I
*
      RETURN
      END
*
*=================== CHTERM ===================================
* 
*  Change temperature/density and scale velocities/coordinates 
*                                                              
	subroutine CHTERM(BOXLN,BOYLN,BOZLN)
	include "prcm.h"
	PTX=0.
	PTY=0.
	PTZ=0.
*  Check exess momenta
	do I=1,NSTOT
	  PTX=PTX+VX(I)
	  PTY=PTY+VY(I)
	  PTZ=PTZ+VZ(I)
	end do	           
	call GETEMP(.false.)
	TEMPO=TEMP
	SUMASD=TOTMAS/UNITM
	DVX=PTX/SUMASD
	DVY=PTY/SUMASD
	DVZ=PTZ/SUMASD
	do I=1,NSTOT
	  VX(I)=VX(I)-DVX/MASSDI(I)
	  VY(I)=VY(I)-DVY/MASSDI(I)
	  VZ(I)=VZ(I)-DVZ/MASSDI(I)
	end do
      call GETEMP(.false.)
	if(dabs(TEMPO-TEMP)/TEMP.gt.0.01.and.TASKID.eq.MAST)then
	  write(*,*)' anomalous excess momenta removed'
	  write(*,*)'old temperature ',TEMPO
        write(*,*)'new temperature ',TEMP
      end if
*  Set new temperature
	if(LCHT)then
*	  SCT=sqrt(TRTEMP/TEMP)
	  do I=1,NSTOT
            ITYP=ITYPE(I)
            SCT=sqrt(TRTEMP/TEMPR(ITYP))
	    VX(I)=VX(I)*SCT
            VY(I)=VY(I)*SCT
	    VZ(I)=VZ(I)*SCT
          end do
	  call GETEMP(.false.)
	  if(TASKID.eq.MAST)write(*,*)' New temperature set to',TEMP
	end if
*  Set new volume/density
	if(LCHP)then
	  IF(RHO.gt.1.d-20) THEN
          IF(BOXLN.le.1.d-20.or.BOYLN.le.1.d-20.or.BOZLN.le.1.d-20)THEN
	      VOL = TOTMAS*1.d27/RHO 
	      VOLO = BOXL*BOYL*BOZL
	      if(LOCT)VOLO=0.5*VOLO
	      SCX = (VOL/VOLO)**(1.0/3.0)
	      SCY=SCX
	      SCZ=SCX
            BOXL        = BOXL*SCX
            BOYL        = BOYL*SCY
            BOZL        = BOZL*SCZ
	      if(TASKID.eq.MAST)
     +      write(*,'(2(a,f10.4))')' New density: ',RHO,' BOXL=',BOXL
          else  ! (BOXLN
	      SCX=BOXLN/BOXL                
	      SCY=BOYLN/BOYL                
	      SCZ=BOZLN/BOZL                
	      BOXL=BOXLN
	      BOYL=BOYLN
	      BOZL=BOZLN
	      VOL=BOXL*BOYL*BOZL   
	      if(LOCT)VOL=0.5*VOL
	      DENS	= TOTMAS*1.d-3/(VOL*UNITL**3)
	      if(TASKID.eq.MAST)
     +      write(*,'(2(a,3f10.4))')'New BOX sizes ',BOXL,BOYL,BOZL,
     +      ' Density',DENS
          ENDIF   ! BOXLN
        ELSE      ! RHO
*Vacuum simulation - arbitrary box size:
          CL        = 1000.D0
	    BOXL=CL
	    BOYL=CL
	    BOZL=CL
	    SCX=1.
	    SCY=1.
	    SCZ=1.
          if(TASKID.eq.MAST)PRINT "(/,'*** VACUUM SIMULATION ***',/)"
        END IF    ! RHO
*  Scaling coordinates
	  N=0
	  DO ITYP  = 1,NTYPES                    ! over types
          ISB      = ISADR (ITYP)+1
          ISE      = ISADR (ITYP +1)
          JBE      = IADDR (ITYP)+1
          JEN      = IADDR (ITYP +1)
          DO J     = JBE,JEN                  ! over molecules
	      DX=(SCX-1.d0)*X(J)
	      DY=(SCY-1.d0)*Y(J)
	      DZ=(SCZ-1.d0)*Z(J)
	      X(J)=X(J)+DX
            Y(J)=Y(J)+DY
            Z(J)=Z(J)+DZ
            DO IS    = ISB,ISE                ! over atoms
              N        = N+1
              SX(N)=SX(N)+DX
              SY(N)=SY(N)+DY
              SZ(N)=SZ(N)+DZ
            END DO! OF IS
          end do ! of J
	  end do ! of ITYP 
	  if(TASKID.eq.MAST)write(*,*)' Scaling distances by ',SCX,SCY,SCZ
	  call RECLEN(1.d0,1.d0,1.d0)
	end if  ! LCHP   
	return
	end
*
*||||||||||||||   RUN-TIME |||||||||||||||||||||||||||||||
*
*
*=============== ZEROFS =============================================
*  
*    Set some arrays and variables to 0
*
      SUBROUTINE ZEROFS(NNN)
      include "prcm.h"

*Zero forces:
*
      GO TO (1000,2000),NNN
*
 1000 CONTINUE
      PELS=0.
	VIRS=0.
	VIRB=0.
	VIRO=0.
	VIRT=0.
	VIRA(1)=0.
	VIRA(2)=0.
	VIRA(3)=0.
	WIRSS=0.
	VIRLS=0.
	VIRFX=0.
	VIRFY=0.
	VIRFZ=0.
      PINT	= 0.
	EINT	= 0.          
      PE         = 0.D0
      DO I       = 1,MOLINT
      POTES(I)   = 0.D0
      POTLJ(I)   = 0.D0
      END DO
	do I=1,NTYPES
	SELFPE(I)      = 0.D0
      PES14 (I)      = 0.D0
      PSR14 (I)      = 0.D0
      end do
      DO I       = 1,NSTOT
      HX(I)      = 0.D0
      HY(I)      = 0.D0
      HZ(I)      = 0.D0
      END DO
      RETURN
*
 2000 CONTINUE
*
      VIR=0.
      VIRL=0.
      VIRN=0.
      WIRF=0.
	WIRS=0.
	VIR1=0.
	PE1=0.
	PELS1=0.
	VIRX=0.
	VIRY=0.
	VIRZ=0.
	DO I       =1,NSTOT
      GX(I)      = 0.D0
      GY(I)      = 0.D0
      GZ(I)      = 0.D0
      END DO
      RETURN
      END
*
*=============== GETCOM ============================================
*                                      
*    Calculate centres of mass of molecules
*
      SUBROUTINE GETCOM
      include "prcm.h"
*
      N           = 0
      I           = 0
      DO ITYP     = 1,NTYPES
      NSBEG       = ISADR  (ITYP)+1
      NSEND       = ISADR  (ITYP +1)
      SUMM        = SUMMAS (ITYP)
      DO J        = 1,NSPEC(ITYP)
      I           = I+1
      X (I)       = 0.D0
      Y (I)       = 0.D0
      Z (I)       = 0.D0 
	PX (I)       = 0.D0
      PY (I)       = 0.D0
      PZ (I)       = 0.D0
Calculate C.O.M. vectors:
      DO IS       = NSBEG,NSEND
      N           = N+1
      X (I)       = X(I)+MASS(IS)*SX(N)
      Y (I)       = Y(I)+MASS(IS)*SY(N)
      Z (I)       = Z(I)+MASS(IS)*SZ(N)
      PX (I)       = PX(I)+VX(N)
      PY (I)       = PY(I)+VY(N)
      PZ (I)       = PZ(I)+VZ(N)
      END DO! OF IS
      X (I)       = X(I)/SUMM
      Y (I)       = Y(I)/SUMM
      Z (I)       = Z(I)/SUMM
      END DO! OF I
      END DO! OF ITYP
*
      N           = 0
      I           = 0
      DO ITYP     = 1,NTYPES
      NSBEG       = ISADR  (ITYP)+1
      NSEND       = ISADR  (ITYP +1)
      DO J        = 1,NSPEC(ITYP)
      I           = I+1
      DO IS       = NSBEG,NSEND
      N           = N+1
Calculate short site vectors
      WX(N)       = SX(N)-X(I)
      WY(N)       = SY(N)-Y(I)
      WZ(N)       = SZ(N)-Z(I)
      END DO! OF IS
      END DO! OF I
      END DO! OF ITYP
*Apply periodic boundaries on COM points
      DO I        = 1,NOP
        call PBC(X(I),Y(I),Z(I))
      END DO! OF I
*
      N           = 0
      I           = 0
      DO ITYP     = 1,NTYPES
      NSBEG       = ISADR  (ITYP)+1
      NSEND       = ISADR  (ITYP +1)
      DO J        = 1,NSPEC(ITYP)
      I           = I+1
      DO IS       = NSBEG,NSEND
      N           = N+1
*Reconstruct long site vectors
      SX(N)       = WX(N)+X(I)
      SY(N)       = WY(N)+Y(I)
      SZ(N)       = WZ(N)+Z(I)
      END DO! OF IS
      END DO! OF I
      END DO! OF ITYP
*
      RETURN
      END
*
*=============== DIPOLE ===========================================
*                                                                
*   Calculate dipole moment
*
      SUBROUTINE DIPOLE
      include "prcm.h"
*
      FAK	  = 1.0/0.52917
      FAC         = 2.5418
      N           = 0
      DO ITYP     = 1,NTYPES
      NSP         = NSPEC(ITYP)
	NSS	    = NSITS(ITYP) 
      FNSPI       = 1.D0/DFLOAT(NSP)
      DIPMOM      = 0.D0
      ISB         = ISADR(ITYP)+1
      ISE         = ISADR(ITYP +1)
      IBE         = IADDR(ITYP)+1
      IEN         = IADDR(ITYP +1)
      DO I        = IBE,IEN
	if(NSS.gt.1)then
      DMX         = 0.D0
      DMY         = 0.D0
      DMZ         = 0.D0
      DO IS       = ISB,ISE
      N           = N+1
Calculate permanent dipole moment (magnitude)
      DMX         = DMX+CHARGE(IS)*WX(N)
      DMY         = DMY+CHARGE(IS)*WY(N)
      DMZ         = DMZ+CHARGE(IS)*WZ(N)
      END DO! OF IS
      DNORM       = DSQRT(DMX**2+DMY**2+DMZ**2)
      DIPMOM      = DIPMOM+DNORM*FAC*FAK*RF
      IF(DNORM.GT.1.d-10) then
      DPX(I)      = DMX/DNORM
      DPY(I)      = DMY/DNORM
      DPZ(I)      = DMZ/DNORM
	else
	DPX(I)=0.d0
	DPY(I)=0.d0
	DPZ(I)=0.d0
	end if   
	else
	DIPMOM=0.d0
	end if
      END DO! OF I
      DIPMOM      = DIPMOM*FNSPI
      IF(MOD(NSTEP,10).EQ.0.and.IPRINT.ge.8) THEN
      IF(DABS(DIPMOM).GT.0.00001D0) THEN
      PRINT "(' DIPOLE MOMENT(D):',F8.3,' FOR TYPE  ',A6)"
     X,DIPMOM,NAME(ITYP)
      END IF
      END IF          
      END DO! OF ITYP
      RETURN
      END                                  
*
*=============== MPOLES =============================================
*
      SUBROUTINE MPOLES(SITE,Q,DM,QM,OM,HM,NSB,NSE,NS)
      IMPLICIT real*8 (A-H,O-Z)
*
      DIMENSION Q(NS),SITE(NS,3)
     X,DM(3),QM(3,3),OM(3,3,3),HM(3,3,3,3)
*
      FAK		= 1.0/0.52917
      DO 10 IS		= NSB,NSE
      DO 10 J		= 1,3
      DM(J) 		= 0.D0
      DO 10 K		= 1,3
      QM(J,K)		= 0.D0
      DO 10 L		= 1,3
      OM(J,K,L)		= 0.D0
      DO 10 M		= 1,3
      HM(J,K,L,M)	= 0.D0
   10 CONTINUE
*
      DO 20 IS		= NSB,NSE
      DO 20 J		= 1,3
      DM(J) 		= DM(J)+Q(IS)*SITE(IS,J)*FAK
      DO 20 K		= 1,3
      QM(J,K)		= QM(J,K)+Q(IS)*SITE(IS,J)*SITE(IS,K)*FAK**2
      DO 20 L		= 1,3
      OM(J,K,L)		= OM(J,K,L)+Q(IS)*SITE(IS,J)*
     X                    SITE(IS,K)*SITE(IS,L)*FAK**3
      DO 20 M		= 1,3
      HM(J,K,L,M)	= HM(J,K,L,M)+Q(IS)*SITE(IS,J)*
     X                    SITE(IS,K)*SITE(IS,L)*SITE(IS,M)*FAK**4
   20 CONTINUE
      RETURN
      END

*....
*======== XINTGR =======================================================
*....
      FUNCTION XINTGR(H,Y,NDIM)
      IMPLICIT REAL*8 (A-H,O-Z)
      DIMENSION Y(*)
      HT=H/3.0 
	XINTGR=0.d0
      IF(NDIM.LT.6)RETURN
      SUM1=4.00*Y(2)
      SUM1=HT*(Y(1)+SUM1+Y(3))
      AUX1=4.00*Y(4)
      AUX1=SUM1+HT*(Y(3)+AUX1+Y(5))
      AUX2=HT*(Y(1)+3.8750*(Y(2)+Y(5))+2.6250*(Y(3)+Y(4))+Y(6))
      SUM2=4.00*Y(5)
      SUM2=AUX2-HT*(Y(4)+SUM2+Y(6))
      AUX=4.00*Y(3)
      IF(NDIM-6)5,5,2
    2 DO 4 I=7,NDIM,2
      SUM1=AUX1
      SUM2=AUX2
      AUX1=4.00*Y(I-1)
      AUX1=SUM1+HT*(Y(I-2)+AUX1+Y(I))
      IF(I-NDIM)3,6,6
    3 AUX2=4.00*Y(I)
      AUX2=SUM2+HT*(Y(I-1)+AUX2+Y(I+1))
    4 CONTINUE
    5 CONTINUE
      XINTGR=AUX2
      RETURN
    6 CONTINUE
      XINTGR=AUX1
      RETURN
      END
*	
*=============== GETKIN ================================================
*
      SUBROUTINE GETKIN
      include "prcm.h"
*
      real*8 UX(NTOT),UY(NTOT),UZ(NTOT)
	real*8 TOR(9),TORI(9) 
*                    
      N      = 0
	EKTR	= 0.
	EKROT	= 0.
	TKET	= 0.
      DO ITYP  = 1,NTYPES                    ! over types
	  SUMMD	= SUMMAS(ITYP)/(AVSNO*1000.0*UNITM)
        NSS      = NSITS (ITYP)
        NSP      = NSPEC (ITYP)
        ISB      = ISADR (ITYP)+1
        ISE      = ISADR (ITYP +1)
        JBE      = IADDR (ITYP)+1
        JEN      = IADDR (ITYP +1)
        DO J     = JBE,JEN                  ! over molecules
          PX(J)    = 0.D0
          PY(J)    = 0.D0
          PZ(J)    = 0.D0
          QX(J)    = 0.D0
          QY(J)    = 0.D0
          QZ(J)    = 0.D0
	    do II=1,9
	      TOR(II)=0.d0
	    end do
          DO IS    = ISB,ISE                ! over atoms
            N      = N+1
*
*	Molecular linear momenta:
*
            PX(J)  = PX(J)+ VX(N)
            PY(J)  = PY(J)+ VY(N)
            PZ(J)  = PZ(J)+ VZ(N)
	    TKET = TKET+0.5*(VX(N)**2+VY(N)**2+VZ(N)**2)*MASSDI(N)
*
*	Molecular angular momenta:
*
            QX(J)    = QX(J)+(VY(N)*WZ(N)-WY(N)*VZ(N))
            QY(J)    = QY(J)+(VZ(N)*WX(N)-WZ(N)*VX(N))
            QZ(J)    = QZ(J)+(VX(N)*WY(N)-WX(N)*VY(N))
	      TOR(1)=TOR(1)+MASSD(IS)*(WY(N)**2+WZ(N)**2)
	      TOR(5)=TOR(5)+MASSD(IS)*(WX(N)**2+WZ(N)**2)
	      TOR(9)=TOR(9)+MASSD(IS)*(WX(N)**2+WY(N)**2)  
	      TOR(2)=TOR(2)-MASSD(IS)*WX(N)*WY(N)
	      TOR(3)=TOR(3)-MASSD(IS)*WX(N)*WZ(N)
	      TOR(6)=TOR(6)-MASSD(IS)*WY(N)*WZ(N)
          END DO! OF IS 
	    TOR(4)=TOR(2)
	    TOR(7)=TOR(3)
	    TOR(8)=TOR(6) 
	    call INV3B3(TOR,TORI,DET,IER)
	    EKTR = EKTR+0.5*(PX(J)**2+PY(J)**2+PZ(J)**2)/SUMMD
          RXJ    = 0.D0
          RYJ    = 0.D0
          RZJ    = 0.D0 
	    N	= N-NSS
          DO IS    = ISB,ISE
            N      = N+1
            FAC=MASS(IS)/SUMMAS(ITYP)
* local atomic momenta
            VTX = VX(N)-PX(J)*FAC
            VTY = VY(N)-PY(J)*FAC
            VTZ = VZ(N)-PZ(J)*FAC
* local angular molecular momenta
            RXJ    = RXJ+(VTY*WZ(N)-WY(N)*VTZ)
            RYJ    = RYJ+(VTZ*WX(N)-WZ(N)*VTX)
            RZJ    = RZJ+(VTX*WY(N)-WX(N)*VTY)
          END DO! OF IS    
* rotational kinetic energy
	    if(IER.eq.0)then
	      EROTM=0.5*(TORI(1)*RXJ**2+(TORI(2)+TORI(4))*RXJ*RYJ+
     +                 TORI(5)*RYJ**2+(TORI(3)+TORI(7))*RXJ*RZJ+
     +                 TORI(9)*RZJ**2+(TORI(6)+TORI(8))*RYJ*RZJ)
	    else
	      ROTM=0.
            PP=RXJ**2+RYJ**2+RZJ**2
            if(PP.lt.1.d-50)go to 100
	      N	= N-NSS
            DO IS    = ISB,ISE
              N        = N+1
              SS=WX(N)**2+WY(N)**2+WZ(N)**2
              SP=WX(N)*RXJ+WY(N)*RYJ+WZ(N)*RZJ
              ROTM=ROTM+MASSD(IS)*(SS*PP-SP**2)
            end do
            if(ROTM.ge.1.d-50)then
	        EROTM=0.5d0*PP**2/ROTM
	      else
	        EROTM=0.d0
	      end if
	    end if
	    EKROT=EKROT+EROTM
          TROTM        = EROTM*ENERF*TEMPF
	  if(IPRINT.ge.10)
     +  write(*,'(2i5,e14.5,f10.3)')J,IER,EROTM,TROTM
 100	  continue
	end do ! of J
      end do ! of ITYP
      EKINT=TKET-EKTR-EKROT
      TTR         = EKTR*ENERF*TEMPF/(FNOP-1.d0)
      if(FNSTR.gt.0.01)then
          TROT        = 3.*EKROT*ENERF*TEMPF/FNSTR
	else
	  TROT=TEMP
	end if 
      if(FNSTI.gt.0.01)then
	  TINT	= 3.d0*EKINT*ENERF*TEMPF/FNSTI
	else
	  TINT=TEMP
	end if
*	TTTT= 3.d0*TKET*ENERF*TEMPF/FNST
*	WW= 0.001*ENERF/FNOP 
*	write(*,'(a5,4f12.4)')'kien:',TKET*WW,EKTR*WW,EKROT*WW,EKINT*WW
*	write(*,'(a5,4f12.4)')'temp:',TTTT,TTR,TROT,TINT
	RETURN
      END
*
*=============== GETMOI ========================================
*

      SUBROUTINE GETMOI(X,Y,Z,M,MOMENT,NSS)
      IMPLICIT real*8 (A-H,O-Z)
      real*8 M,MOMENT
      DIMENSION X(NSS),Y(NSS),Z(NSS),M(NSS)
      DIMENSION MOMENT(3,3),R(3)
*
      DO INERT           = 1,3
      DO IA              = 1,3
      TT                 = 0.D0
      MOMENT(INERT,IA)   = 0.D0
      IF(INERT.EQ.IA) TT = 1.D0
      DO I               = 1,NSS
      R(1)               =  X(I)
      R(2)               =  Y(I)
      R(3)               =  Z(I)
      SQS                =  X(I)**2+Y(I)**2+Z(I)**2
      MOMENT(INERT,IA)   = MOMENT(INERT,IA)+M(I)*(TT*SQS-R(INERT)*R(IA))
      END DO! OF I
      END DO! OF IA
      END DO! OF INERT
*
      RETURN
      END
*
*===================== INV3B3 ==========================================
*
      SUBROUTINE INV3B3(A,AI,DET,IER)
      IMPLICIT real*8 (A-H,O-Z)
      DIMENSION A(9),AI(9)
*
      AI(1) = A(5)*A(9)-A(6)*A(8)
      AI(2) = A(3)*A(8)-A(2)*A(9)
      AI(3) = A(2)*A(6)-A(3)*A(5)
      AI(4) = A(6)*A(7)-A(4)*A(9)
      AI(5) = A(1)*A(9)-A(3)*A(7)
      AI(6) = A(3)*A(4)-A(1)*A(6)
      AI(7) = A(4)*A(8)-A(5)*A(7)
      AI(8) = A(2)*A(7)-A(1)*A(8)
      AI(9) = A(1)*A(5)-A(2)*A(4)
*
      DET   = A(1)*AI(1)+A(4)*AI(2)+A(7)*AI(3)
	SPUR=A(1)+A(5)+A(9)   
	IER=0
	if(dabs(DET).lt.1.d-6*dabs(SPUR**3))IER=1
      IF(DABS(DET).gt.0.D0) then
	  R=1.D0/DET
	else
	  R=0.d0
	  IER=2
	end if
*
      AI(1) = R*AI(1)
      AI(2) = R*AI(2)
      AI(3) = R*AI(3)
      AI(4) = R*AI(4)
      AI(5) = R*AI(5)
      AI(6) = R*AI(6)
      AI(7) = R*AI(7)
      AI(8) = R*AI(8)
      AI(9) = R*AI(9)
*
      RETURN
      END
*
*
* |||||||||||||||||   R D F   ||||||||||||||||||||||||||||||||||||||
*
*================= RDFINP =============================================
*
      SUBROUTINE RDFINP
      include"prcm.h" 
	character*80 str , TAKESTR
*     
	if(TASKID.eq.MAST)then
        PRINT *
        PRINT *,'----------------------------------------'
        PRINT *,'*** PREPARATION FOR RDF CALCULATIONS ***'
        PRINT *,'----------------------------------------'
      end if
	NMGR(0)    = 'NONE    '
	do IS=1,NSITES
	  NGRI(IS)=0
	  do J=1,MXGRS
	    IGRI(IS,J)=0
	    MGRI(IS,J)=0
	  end do
	end do
      IF(ALLSTS) THEN
        N          = 0
        DO I       = 1,NSITES
          DO J       = I,NSITES
            N          = N+1
	      NGRI (I) = NGRI(I)+1
	      if(J.ne.I)NGRI(J)=NGRI(J)+1
	      if(NGRI(I).gt.MXGRS.or.NGRI(J).gt.MXGRS.or.N.gt.MAXGR)then
 	        write(*,*)' List of RDF limited to ',N,' pairs'
	        write(*,*)
     +' To calculate other RDF, specify list of RDF-s explicitly'           
	        go to 21
	      end if 
	      IGRI(I,NGRI(I))=J
	      MGRI(I,NGRI(I))=N
	      IGRI(J,NGRI(J))=I
	      MGRI(J,NGRI(J))=N
          END DO  
        END DO 
*
      ELSE
*     
	if(TASKID.eq.MAST)then
        N          = 0    
	  IE=-1                
	  STR=TAKESTR(IO,IE) 
	  if(IE.lt.0)then
	    MAXRDF=0
	    LGR=.false.
	    write(*,*)' Empty list of RDF'
	    return
	  end if
        READ(STR,*,err=99) MAXRDF
        DO K       = 1,MAXRDF
          N          = N+1  
	    IE=-1
	    STR=TAKESTR(IO,IE)
	    if(IE.lt.0)go to 19
	    if(str(1:1).eq.'&')then
            READ(str(2:24),*)IREP
	      do IR=1,IREP
	        STR=TAKESTR(IO,IE)
	        read(STR,*,err=99)I,J  
	        NGRI(I)=NGRI(I)+1 
	        if(J.ne.I)NGRI(J)=NGRI(J)+1
	        if(NGRI(I).gt.MXGRS)then
 	          write(*,*)' List of RDF exceeded for site ',I,' ',NM(I)
	          write(*,*)' increase MXGRS in prcm.h'
		  write(*,*)' RDF-s input interrupted'          
	          go to 19
	        end if 
	        if(NGRI(J).gt.MXGRS)then
 	          write(*,*)' List of RDF exceeded for site ',J,' ',NM(J)
	          write(*,*)' increase MXGRS in prcm.h'
		  write(*,*)' RDF-s input interrupted'          
	          go to 19
	        end if 
*  NGRI - number of RDF-pairs for this site
*  IGRI - 2-nd site for this RDF
*  MGRI - RDF number
	        IGRI(I,NGRI(I))=J
	        MGRI(I,NGRI(I))=N
	        IGRI(J,NGRI(J))=I
	        MGRI(J,NGRI(J))=N
	      end do
	    else 
	      IPER=1
	      read(str,*,err=99)I,J
	      NGRI(I)=NGRI(I)+1 
	      if(J.ne.I)NGRI(J)=NGRI(J)+1
	      if(NGRI(I).gt.MXGRS)then
 	        write(*,*)' List of RDF exceeded for site ',I,' ',NM(I)
	        write(*,*)' increase MXGRS in prcm.h'
		write(*,*)' RDF-s input interrupted'          
	        go to 19
	      end if 
	      if(NGRI(J).gt.MXGRS)then
 	        write(*,*)' List of RDF exceeded for site ',J,' ',NM(J)
	        write(*,*)' increase MXGRS in prcm.h'
	        write(*,*)' RDF-s input interrupted'          
	        go to 19
	      end if 
	      IGRI(I,NGRI(I))=J
	      MGRI(I,NGRI(I))=N
	      IGRI(J,NGRI(J))=I
	      MGRI(J,NGRI(J))=N
	    end if
        END DO
	end if      
 19	call TRANSRDF(NS,MXGRS,MAST,TASKID,NGRI,IGRI,MGRI)
      END IF                              
*
  21  MAXRDF     = N
      if(TASKID.eq.MAST)
     +PRINT "('NR OF PAIRS IN RDF CALCULATIONS: ',I5)",N
      IF(MAXRDF.GT.MAXGR) STOP '!!! MAXRDF.GT.MAXGR IN RDFINP'
*
      if(TASKID.eq.MAST)
     +PRINT "('*** NR OF RADIAL DISTRIBUTION FUNCTIONS: ',I4)",N
*
      NRDF       = 0
      DO I       = 1,MAXGR
	  IIN(I)=0
	  IGR(I)=0
	  JGR(I)=0
        DO J       = 1,MAX
          IRDF(J,I)  = 0
        END DO
      END DO
*     
*  Check RDF list and calculate normalising factors
      DO ITYP    = 1,NTYPES
        NSPI       = NSPEC(ITYP)
        ISB        = ISADR(ITYP)+1
        ISE        = ISADR(ITYP +1)
        DO IS      = ISB,ISE 
	    NNR=NGRI(IS)
	    do K=1,NNR
	      JS = IGRI(IS,K)
	      if(IS.le.JS)then
		N    = MGRI(IS,K)
		JTYP = ITS(JS)
              NSPJ = NSPEC(JTYP)
              NMGR(N)    = NM(IS)//NM(JS)
              IGR (N)    = NSPI
              JGR (N)    = NSPJ
	        if(IS.eq.JS.and.NSPI.ne.1)then
	          IIN(N)=IIN(N)+(NSPI-1)*NSPI/2
	        else
	          IIN(N)=IIN(N)+NSPI*NSPJ
	        end if     
	      end if
	    end do
*
        END DO! OF IS
      END DO! OF ITYP
*
      DRDF     = RDFCUT/DFLOAT(MAX)
      DRDFI    =  1.D0/DRDF
      if(TASKID.eq.MAST)then
        PRINT "('*** ONE RDF SLICE IN ANGSTROM:  ',F8.5)",DRDF
        PRINT *
	end if 
*  Check RDF table
	if(IPRINT.ge.7)then   
	  write(*,*)' Calculate RDF:'  
	  do IS=1,NSITES
	    NNR=NGRI(IS) 
          do KS=1,NNR
	      JS=IGRI(IS,KS)            
	      K=MGRI(IS,KS)
	    write(*,'(a,2i5,3x,a4,a1,a4,i6,3i9)')
     +' sites ',IS,JS,NM(IS),'-',NM(JS),K,IGR(K),JGR(K),IIN(K)/2
	    end do                                     
	  end do
	end if
*
 22   RETURN      
 99   IE=99
	STR=TAKESTR(IO,IE)
      END                       
*
*================= RDFOUT =============================================
*
      SUBROUTINE RDFOUT(IOUT)
      include"prcm.h"
*
      CHARACTER*12 LABRDF
      DIMENSION GR(MAXS,MAXGR),GRO(MAXS,MAXGR),DF(MAXS)
	integer IRDF1(MAXS,MAXGR)
      DATA JINIT/0/, LABRDF /'rdf:        '/
	save GRO,FNRNR,JINIT,NRNR
*     
      IF(NRDF.EQ.0.and.(.not.LGRST)) RETURN
CM
CM   collecting RDF
CM    
	call ADD_RDF(IRDF,IRDF1,MAXS,MAXGR,MAST)  
	if(TASKID.ne.MAST)RETURN
      if(IOUT.ne.0)then
      PRINT *
      PRINT *,'+++++++++++++++++++++++++++++++++++++++++++++++'
      PRINT *,'***      OUTPUT FROM RDF CALCULATIONS       ***'
      PRINT *,'+++++++++++++++++++++++++++++++++++++++++++++++'
      PRINT *
      PRINT "('*** RDF  COLLECTED  DURING:  ',I6,'   STEPS')",NRDF
      PRINT "('*** NR  OF  RDF TYPES  CALCULATED       ',I6)",MAXRDF
      PRINT "('*** CUT OFF RADIUS DIVIDED TO ',I6,' SLICES')",MAX
      PRINT *
*
      PRINT "(1X,130('='))"
      end if
      FNRDF        = DFLOAT(NRDF)
      if(JINIT.eq.0)then
      NRNR         = 0
      FNRNR        = 0.D0
      DO I         = 1,MAXRDF
      DO J         = 1,MAX
      GRO(J,I)      = 0.D0
      END DO! OF J
      END DO! OF I
      end if
*
      IF(LGRST.and.JINIT.eq.0) THEN
        OPEN (UNIT=20,file=frdf,status='old',FORM='UNFORMATTED',
     +        err=111)
        READ(20,err=111) NRNR
        DO I         = 1,MAXRDF
          READ(20,err=112,end=112) (GRO(J,I),J=1,MAX) 
        END DO
        PRINT "('*** RDF RESTART FOR ',I8,' OLD CONFIGURARTIONS')",NRNR
        FNRNR        = DFLOAT(NRNR)
        close (20)
        JINIT=1
      END IF 
	go to 113
 111  write(*,*)' RDF restart failed. File ',frdf
	write(*,*)' RDF are collected only from this run'
	go to 113
 112	write(*,*)' Invalid RDF restart file ',frdf
	frdf(1:1)='?'
	if(LGDMP)write(*,*)' RDF dump will be in file',frdf 
 113  continue
*
      FNRINV       =  1.D0/(FNRDF+FNRNR)
      if(NRDF.eq.0)then
        FNRDF=1.
      end if
      RDFINC       =  RDFCUT/DFLOAT(MAX)
*
      DO 200 I     = 1,MAXRDF
*
      FNI          = DFLOAT(IGR(I))
      FNJ          = DFLOAT(JGR(I))
	FIJ	     = dfloat(IIN(I)) 
*
      if(IOUT.ne.0)then
      PRINT *
      PRINT "('#*** THIS PAIR: ',A8,'   ---> '
     X       ,'TOTAL NR OF CONFIGURATIONS',I8)",NMGR(I),NRNR+NRDF
      PRINT *
      PRINT "(3X,5HR(A) ,5X,3HRDF,3X,7HINT RDF)"
*
      end if
      AVRNRI       =  FNRDF
      DNSNRI       = VOL/FNI
      DNSNRJ       = VOL/FNJ 
      RDFINT       = 0.D0 
	LABRDF(5:12)=NMGR(I)
*
      M            = 0
      DO 100 J     = 1,MAX
      M            = M+1    
      DIST         = RDFINC*(DFLOAT(J)-0.5)
      SHELLV       = 4.D0*PI*RDFINC*(DIST**2+RDFINC**2/12.D0)
      FIRDF        = DFLOAT(IRDF1(J,I))
      RDFINT       = RDFINT+FIRDF/ (AVRNRI*FNI)
      RDFIJ        = VOL*FIRDF/(FIJ*AVRNRI*SHELLV)
*
      GR(J,I)      = (FNRDF*RDFIJ+FNRNR*GRO(J,I))*FNRINV
      RDFIJ        = GR(J,I)
	RDFINT	     = RDFINT+RDFIJ*SHELLV/DNSNRJ
      DF(M)        =  RDFIJ*DIST**2
      FINT         =  4.D0*PI*XINTGR(RDFINC,DF,M)
      FINTI        =  FINT/DNSNRI
      FINTJ        =  FINT/DNSNRJ
*
      IF(RDFINT.NE.0.D0.AND.DIST.LE.RDFCUT.and.IOUT.ne.0)PRINT 
     X"(F10.4,f11.6,2f9.4,10x,a12,I9)",
     +DIST,RDFIJ,FINTI,FINTJ,LABRDF
  100 CONTINUE
  200 CONTINUE
*
      if(IOUT.ne.0)PRINT "(1X,130('='))"       
	NRDFT=NRDF+NRNR
      IF(LGDMP.and.(.not.linput).and.NRDFT.gt.0) THEN
      OPEN (UNIT   = 21,file=frdf,status='unknown',FORM='UNFORMATTED')
      WRITE(21) NRDFT
      DO I         = 1,MAXRDF
      WRITE(21) (GR(J,I),J=1,MAX)
      END DO! OF I
      PRINT "('*** RDF DUMP FOR ',I8,' CONFIGURARTIONS')",NRNR+NRDF
      close (21)
      END IF
*
      RETURN
      END
*
*=============== GETROT ==========================================
*
      SUBROUTINE GETROT
*
      include"prcm.h"
*
      DIMENSION RX(NS),RY(NS),RZ(NS),FMASS(NS)
      DIMENSION FMOI(3,3)      
      DIMENSION G(3),H(3)
*
      RF       = 1.
      N        = 0
      DO ITYP  = 1,NTYPES
      NSS      = NSITS (ITYP)
      NSP      = NSPEC (ITYP)
      ISB      = ISADR (ITYP)+1
      ISE      = ISADR (ITYP +1)
      JBE      = IADDR (ITYP)+1
      JEN      = IADDR (ITYP +1)
      DO J     = JBE,JEN
      I        = 0
      DO IS    = ISB,ISE
      I        = I+1
      N        = N+1
      RX   (I) = WX(N)
      RY   (I) = WY(N)
      RZ   (I) = WZ(N)
      FMASS(I) = MASSD(IS)
      END DO! OF IS
*
      CALL GETMOI(RX,RY,RZ,FMASS,FMOI,NSS)
*
      CALL HH3BY3(FMOI,G,H)
*
      DO K       = 1,3
      DO L       = 1,3
      RMX(J,K,L) = FMOI(K,L)
      END DO! OF K
      END DO! OF L
*
      END DO! OF J
*
      END DO! OF ITYP
*
      RETURN
      END
*
*========== HH3BY3 ============================================
*
      SUBROUTINE HH3BY3(A,D,E)
*
      IMPLICIT real*8 (A-H,O-Z)
*
      DIMENSION A(3,3),D(3),E(3)
      data ICOUNT/0/
*
      H       = 0.
      SCALE   = ABS(A(3,1))+ABS(A(3,2))
      IF(SCALE.EQ.0) THEN
      E(3)    = A(3,2)
      ELSE
      A(3,1)  = A(3,1)/SCALE
      A(3,2)  = A(3,2)/SCALE
      H       = A(3,1)**2+A(3,2)**2+H
      F       = A(3,2)
      G       =-SIGN(SQRT(H),F)
      E(3)    = SCALE*G
      H       = H-F*G
      A(3,2)  =   F-G
*
      A(1,3)  = A(3,1)/H
      G       = A(1,1)*A(3,1)+A(2,1)*A(3,2)
      E(1)    = G/H
      F       = E(1)*A(3,1)
*
      A(2,3)  = A(3,2)/H
      G       = A(2,1)*A(3,1)+A(2,2)*A(3,2)
      E(2)    = G/H
      F       = F+E(2)*A(3,2)
*
      HH      = F/(H+H)
      F       = A(3,1)
      G       = E(1)-HH*F
      E(1)    = G
      A(1,1)  = A(1,1)-F*E(1)-G*A(3,1)
      F       = A(3,2)
      G       = E(2)-HH*F
      E(2)    = G
      A(2,1)  = A(2,1)-F*E(1)-G*A(3,1)
      A(2,2)  = A(2,2)-F*E(2)-G*A(3,2)
      END IF
      D(3)    = H
*
      SCALE   = 0.
      E(2)    = A(2,1)
      D(2)    = 0.
*
      E(1)    = 0.
      D(1)    = A(1,1)
      A(1,1)  = 1.
      IF(D(2).NE.0) THEN
      G       = A(2,1)*A(1,1)
      A(1,1)  = A(1,1)-A(1,2)*G
      END IF
      D(2)    = A(2,2)
      A(2,2)  = 1.
      A(2,1)  = 0.
      A(1,2)  = 0.
      IF(D(3).NE.0) THEN
      G       = A(3,1)*A(1,1)+A(3,2)*A(2,1)
      A(1,1)  = A(1,1)-A(1,3)*G
      A(2,1)  = A(2,1)-A(2,3)*G
      G       = A(3,1)*A(1,2)+A(3,2)*A(2,2)
      A(1,2)  = A(1,2)-A(1,3)*G
      A(2,2)  = A(2,2)-A(2,3)*G
      END IF
      D(3)    = A(3,3)
      A(3,3)  = 1.
      A(3,1)  = 0.
      A(1,3)  = 0.
      A(3,2)  = 0.
      A(2,3)  = 0.
*
*Diagonalize it:
*
      E(1)    = E(2)
      E(2)    = E(3)
      E(3)    = 0.
      DO 15 L = 1,3
      ITER    = 0
    1 CONTINUE
      DO 12 M = L,2
      DD      = ABS(D(M))+ABS(D(M+1))
      IF(ABS(E(M))+DD.EQ.DD) GO TO 2
   12 CONTINUE
      M       = 3
    2 CONTINUE
      IF(M.NE.L) THEN
      IF(ITER.EQ.30) then
        write(*,*) '!!! TOO MANY ITERATIONS!    in HH3BY3'
        ICOUNT=ICOUNT+1
        if(ICOUNT.gt.1000)stop
        return
      end if
      ITER    = ITER+1
      G       = (D(L+1)-D(L))/(2.*E(L))
      R       = SQRT(G**2+1.)
      G       = D(M)-D(L)+E(L)/(G+SIGN(R,G))
      S       = 1.
      C       = 1.
      P       = 0.
      DO 14 I = M-1,L,-1
      F       = S*E(I)
      B       = C*E(I)
      IF(ABS(F).GE.ABS(G)) THEN
      C       = G/F
      R       = SQRT(C**2+1.)
      E(I+1)  = F*R
      S       = 1./R
      C       = C*S
      ELSE
      S       = F/G
      R       = SQRT(S**2+1.)
      E(I+1)  = G*R
      C       = 1./R
      S       = S*C
      END IF
      G       = D(I+1)-P
      R       =(D(I)-G)*S+2.*C*B
      P       = S*R
      D(I+1)  = G+P
      G       = C*R-B
      F       = A(1,I+1)
      A(1,I+1)= S*A(1,I)+C*F
      A(1,I)  = C*A(1,I)-S*F
      F       = A(2,I+1)
      A(2,I+1)= S*A(2,I)+C*F
      A(2,I)  = C*A(2,I)-S*F
      F       = A(3,I+1)
      A(3,I+1)= S*A(3,I)+C*F
      A(3,I)  = C*A(3,I)-S*F
   14 CONTINUE
      D(L)    = D(L)-P
      E(L)    = G
      E(M)    = 0.
      GO TO 1
      END IF
   15 CONTINUE
      RETURN
      END
*
*========= ROTMAT =================================================
*
      SUBROUTINE ROTMAT(XX,YY,ZZ,IBG,N,NBTOP)
*
      include "prcm.h"
*
      DIMENSION XX(*),YY(*),ZZ(*)
      LOGICAL NBTOP
*
      IBEG  = IBG
      IEND  = IBG+N-1
*
      IF(NBTOP) THEN	! from box to principal:
      DO I  = IBEG,IEND
      AQ    = XX(I)
      BQ    = YY(I)
      CQ    = ZZ(I)
      XX(I) = RMX(I,1,1)*AQ+RMX(I,2,1)*BQ+RMX(I,3,1)*CQ
      YY(I) = RMX(I,1,2)*AQ+RMX(I,2,2)*BQ+RMX(I,3,2)*CQ
      ZZ(I) = RMX(I,1,3)*AQ+RMX(I,2,3)*BQ+RMX(I,3,3)*CQ
      END DO! OF I
      ELSE 		! from principal to box:
      DO I  = IBEG,IEND
      AQ    = XX(I)
      BQ    = YY(I)
      CQ    = ZZ(I)
      XX(I) = RMX(I,1,1)*AQ+RMX(I,1,2)*BQ+RMX(I,1,3)*CQ
      YY(I) = RMX(I,2,1)*AQ+RMX(I,2,2)*BQ+RMX(I,2,3)*CQ
      ZZ(I) = RMX(I,3,1)*AQ+RMX(I,3,2)*BQ+RMX(I,3,3)*CQ
      END DO! OF I
      END IF
*
      RETURN
      END
*
*============ FMELT ================================================
*
      SUBROUTINE FMELT(X0,Y0,Z0,NSP)
      IMPLICIT real*8  (A-H,O-Z)
      DIMENSION X0(*),Y0(*),Z0(*)
      DATA PI/3.1415926536D0/
      CN=PI*(2*NSP)**(1.D0/3.D0)
      AB=0.D0
      BC=0.D0
      CD=0.D0
      DE=0.D0
      EF=0.D0
      FG=0.D0
      DO 10 I=1,NSP
      AB=AB+DCOS(CN*X0(I))
      BC=BC+DSIN(CN*X0(I))
      CD=CD+DCOS(CN*Y0(I))
      DE=DE+DSIN(CN*Y0(I))
      EF=EF+DCOS(CN*Z0(I))
      FG=FG+DSIN(CN*Z0(I))
   10 CONTINUE
      FMLT=(AB*AB+BC*BC+CD*CD+DE*DE+EF*EF+FG*FG)/DFLOAT(3*NSP)
      PRINT "(/1X,'  *****   MELTING FACTOR: ',F12.4/)",FMLT
      RETURN
      END
*
*======================== TESTCONF ===================================
*
	subroutine TESTCONF
*  Check configuration
	include "prcm.h"
	logical OK,OKB
	OK=.true.     
	OKB=.true.
*	if(TASKID.ne.MAST)return
	do I=1,NSTOT-1 
	  IMOL=NNUM(I)
	  ISB =NSITE(I)
	  ITYP=ITYPE(I)
	  do J=I+1,NSTOT
	    JMOL=NNUM(J)
	    JSB =NSITE(J)
	    JTYP=ITYPE(J)
	    DX=SX(I)-SX(J)
	    DY=SY(I)-SY(J)
	    DZ=SZ(I)-SZ(J)
	    call PBC (DX,DY,DZ)
	    RR=sqrt(DX**2+DY**2+DZ**2)
	    if(RR.le.0.6*(SIGMA(ISB)+SIGMA(JSB)).or.RR.le.1.)then  
	      if(IMOL.eq.JMOL)then
*  Check that these are non bound
	        do K=1,NNBB(I) 
*  bound
		  if(INBB(I,K).eq.J)go to 125
                  if(INBB(I,K).eq.-J.and..not.L14NB(ITYP))go to 125
	  	end do
              end if
              if(OK)then
		  OK=.false.
		  write(*,*)'!!! BAD CONFIGURATION   node ',TASKID
		  write(*,*)'The following atom pair are too close'
		  write(*,*)
     +' N1  mol1  site1   N2  mol2  site2    RR        Eel        Elj'
              end if
  	      QIJ=COULF*CHARGE(ISB)*CHARGE(JSB)
              EPSI       =   EPSIL(ISB)*EPSIL(JSB)
              SIGM       =   SIGMA(ISB)+SIGMA(JSB)
              A6        = -EPSI*SIGM**6
              B12       =  EPSI*SIGM**12
              EEL=0.001*ENERF*QIJ/RR
              ELJ=0.001*ENERF*(A6/RR**6+B12/RR**12)
              write(*,'(2(I5,1x,a6,1x,a4),f10.4,2e12.4)')
     +I,NAME(ITYP),NM(ISB),J,NAME(JTYP),NM(JSB),RR,EEL,ELJ 
	    end if
 125	    continue
	  end do
	end do
*   Bonds
	do ITYP=1,NTYPES
        NBBEG     = IADB(ITYP)
        NBEND     = NBBEG+NRB(ITYP)-1   
	  do IIS=NBBEG,NBEND
	    do IM=1,NSPEC(ITYP)      
	      ISHIFT=ISADDR(ITYP)+(IM-1)*NSITS(ITYP)
	      I=ISHIFT+IB(IIS)
	      J=ISHIFT+JB(IIS)           
C	write(*,*)' check ',ITYP,IIS,IM,IB(IIS),JB(IIS),I,J
            DX      = SX(I)-SX(J)
            DY      = SY(I)-SY(J)
            DZ      = SZ(I)-SZ(J)
	      call PBC(DX,DY,DZ)
            RR      = sqrt(DX*DX+DY*DY+DZ*DZ) 
	      REQ     = RB(IIS)
	      if(RR.le.0.7*REQ.or.RR.gt.1.5*REQ)then
	        if(OKB)then
	          OKB=.false.
	          write(*,*)'!!! BAD BOND LENGTH:'
	          write(*,*)
     +' Type  Mol     site1       site2      RR      Req        Ebond'
	        end if
	        ISB = ISADR(ITYP)+IB(IIS)
	        JSB = ISADR(ITYP)+JB(IIS)
	        FF=FB(IIS)*(RR-REQ)**2/EFACT
                write(*,'(a6,i5,2(2x,a4,i5),2f10.4,e13.5)')
     +        NAME(ITYP),IM,NM(ISB),ISB,NM(JSB),JSB,RR,REQ,FF
	      end if
	    end do
	  end do
	end do
	if(OK.and.OKB)write(*,*)' Configuration OK    Node ',TASKID
	return
	end
*
*======================= PBC ========================================
* 
*    Periodic boundary conditions
*
      subroutine PBC(XX,YY,ZZ)
      include "prcm.h"
      if(XX.gt. HBOXL)XX=XX-BOXL
      if(XX.lt.-HBOXL)XX=XX+BOXL 
	if(LHEX)then
C  hexagonal periodic cell - minimum image up to 1.5*BOXL from the center    
	XY=BOXYC*XX
        if(YY.gt.BOXY3-XY.and.(XX.gt.0..or.YY.gt.2.*BOXY3-XY))then
          YY=YY-BOYL
          XX=XX-HBOXL
	  XY=BOXYC*XX
        end if
        if(YY.lt.-BOXY3-XY.and.(XX.lt.0..or.YY.lt.-2.*BOXY3-XY))then
          YY=YY+BOYL
          XX=XX+HBOXL
	  XY=BOXYC*XX
        end if
        if(YY.gt.BOXY3+XY)then
          YY=YY-BOYL
          XX=XX+HBOXL
	  XY=BOXYC*XX
        end if
        if(YY.lt.-BOXY3+XY)then
          YY=YY+BOYL
          XX=XX-HBOXL
        end if
	else                   
C  rectangular cell
        if(YY.gt. HBOYL)YY=YY-BOYL
        if(YY.lt.-HBOYL)YY=YY+BOYL
	end if
      if(ZZ.gt. HBOZL)ZZ=ZZ-BOZL
      if(ZZ.lt.-HBOZL)ZZ=ZZ+BOZL
      if(.not.LOCT)return
C  truncated octahedron abs(x)+abs(y)+abs(z) < 0.75*BOXL
	CORSS=HBOXL*int((4./3.)*(abs(XX)+abs(YY)+abs(ZZ))/BOXL)
	XX=XX-sign(CORSS,XX)
	YY=YY-sign(CORSS,YY)
	ZZ=ZZ-sign(CORSS,ZZ)
	return
      end 
*
*=============== GROUPMOL ===================================
*
      subroutine GROUPMOL
C   This procedure is to gather atoms of a molecule so that the molecule
C   was connected even without periodical boundary conditions
C   (if by some reason atoms of a molecule occured in different simulation 
C   cell and linked only due to periodic boundary conditions)
C
C   The procedure goes over all the bonds in the molecule and check that
C   both atoms of the bond are in the same cell (i.e. they are not
C   linked through the cell boundary by the periodic boundary conditions)
C   The only exception is if in the initial state (defined in the .mol file)
C   the bond is linked through the boundary 
C   (as e.g in periodical DNA molecule)
C
C   This is done to ensure that the whole molecule is always connected, 
C   that is nesessary to calculate correctly center-of mass.
C   Normally the program maintain this condition during the simulation
C   (see subroutine GETCOM), that is why this subroutine is applied only
C   at startup time 
C      
      include "prcm.h"
      logical MOVED(NTOT),BOUND(NTOT)
*
*   1. Cycle over molecules
*   -----------------------
      N = 0
      IMOL = 0
      DO ITYP     = 1,NTYPES
        NSBEG       = ISADR  (ITYP)+1
        NSEND       = ISADR  (ITYP +1)
        SUMM        = SUMMAS (ITYP)
        DO J        = 1,NSPEC(ITYP)
          IMOL           = IMOL+1
          do IS = NSBEG,NSEND
             MOVED(IS)=.false.
             BOUND(IS)=.false.
          end do
*
*   2. Cycle over bonds within the molecule
*   ---------------------------------------
          IBBEG = IADB(ITYP)
          IBEND = IBBEG+NRB(ITYP)-1
          ISHF  = ISADDR(ITYP)+NSITS(ITYP)*(IMOL-IADDR(ITYP)-1)
          IS0 = IB(1)
          MOVED(IS0)=.true.
C  It is not garantied that the molecule would be gathered by a single
C  pass through the bonds. NCOUNT counts number of bonds which
C  satisfy condition of connectivity
 10       NCOUNT=0
          do ISB = IBBEG,IBEND
C  Total site number
             IS = IB(ISB)
             JS = JB(ISB)
C  Length of the bond in "reference" state (in the .mol file)
             RREF=sqrt((R(IS,1)-R(JS,1))**2+(R(IS,1)-R(JS,2))**2+
     +                 (R(IS,3)-R(JS,3))**2)
C  Atom number
             ISP = IS+ISHF
             JSP = JS+ISHF
C  if both IS and JS sites are MOVED, the condition of connectivity
C  is fulfilled for this bond
C  Condition RREF.gt.HBOXL Check that reference conformation is not 
C  linked through the cell boundary
             if((MOVED(IS).and.MOVED(JS)).or.RREF.gt.HBOXL)then
               NCOUNT=NCOUNT+1
C  Link JS atom to IS
             else if(MOVED(IS).and..not.MOVED(JS))then
               DX=SX(JSP)-SX(ISP)
               DY=SY(JSP)-SY(ISP)
               DZ=SZ(JSP)-SZ(ISP)
               call PBC(DX,DY,DZ)
               SX(JSP)=SX(ISP)+DX
               SY(JSP)=SY(ISP)+DY
               SZ(JSP)=SZ(ISP)+DZ
               MOVED(JS)=.true.
               NCOUNT=NCOUNT+1
C  Link IS atom to IS
             else if(.not.MOVED(IS).and.MOVED(JS))then
               DX=SX(ISP)-SX(JSP)
               DY=SY(ISP)-SY(JSP)
               DZ=SZ(ISP)-SZ(JSP)
               call PBC(DX,DY,DZ)
               SX(ISP)=SX(JSP)+DX
               SY(ISP)=SY(JSP)+DY
               SZ(ISP)=SZ(JSP)+DZ
               MOVED(IS)=.true.
               NCOUNT=NCOUNT+1
             end if  ! if(MOVED(IS ...
           end do
C   Check that all bonds are connected 
C   (if not, repeat the cycle over bonds)
           if(NCOUNT.lt.NRB(ITYP))go to 10
         end do
       end do
       return
       end


